; ***************************************************************************
; * NASM Prime Classification Console Program                               *
; * Author : Fabien Sibenaler                                               *
; * Date   : April 2007                                                     *
; ***************************************************************************
; * New version 2012 : 														                          *  
; * new computation for the                                                 * 
;	*			limit of the level = jump - 1                                       *
;	*			limit of weight    = sqrt(2P(n) -P(n+1))	                          * 
; * console dimensions - colors - title                                     *    
; * manage a file for the results                                           *
; ***************************************************************************
; * Prime Classification by Rmi Eismann                                    *
; * http://reismann.free.fr/                                                *
; *************************************************************************** 

; Assemble with: nasmw -f obj -O3 k_classification_w32qw_ve.asm
;		-lC_k_classification.lst : generate listing (if you like)
;
; Include file may be put here with
;   -p win32n.inc
; if you choose to use it

; Link with: ALink l_classification_w32qw_ve.obj -c -oPE -m+ -subsys console
;   -c+ : enable case sensitive linking
;   -oPE : Windows PE executable
;   -m+ : generate map file (if you like)
;   -subsys console : generate a console program

; *************
; * EXTERNALS *
; *************
extern CreateFileA ; create new file
;    pFilename:PTR BYTE,     ; ptr to filename
;    accessMode:dword,       ; access mode
;    shareMode:dword,        ; share mode
;    lpSecurity:dword,       ; can be NULL
;    howToCreate:dword,      ; how to create the file
;    attributes:dword,       ; file attributes
;    htemplate:dword         ; handle to template file
;Returns: (HANDLE) eax = Handle if successful,
;                        INVALID_HANDLE_VALUE if fails.
import CreateFileA c:\Windows\system32\Kernel32.dll

extern WriteFile             ; write buffer to output file
;    fileHandle:dword,        ; output handle
;    pBuffer:PTR BYTE,        ; pointer to buffer
;    nBufsize:dword,          ; size of buffer
;    pBytesWritten:PTR dword, ; number of bytes written
;    pOverlapped:PTR dword    ; ptr to asynchronous info
;Returns: (BOOL) eax = TRUE (non-zero) if successful,
;                      FALSE (zero) if fails.
import WriteFile c:\Windows\system32\Kernel32.dll

extern ReadConsoleA
import ReadConsoleA c:\Windows\system32\Kernel32.dll
		;	 hConsoleInput
		;	[in] Handle to the console input buffer. The handle must have GENERIC_READ access.
		;	 lpBuffer
		;	[out] Pointer to a buffer that receives the data read from the console input buffer.	
		;	 nNumberOfCharsToRead
		;	[in] Specifies the number of TCHARs to read. Because the function can read either Unicode or ANSI characters,
		; the size of the buffer pointed to by the lpBuffer parameter should be 
		;at least nNumberOfCharsToRead * sizeof(TCHAR) bytes.	
		;	 lpNumberOfCharsRead
		;	[out] Pointer to a variable that receives the number of TCHARs actually read.	
		;	 lpReserved
		;	[in] Reserved; must be NULL.

extern GetTickCount
import GetTickCount c:\Windows\system32\Kernel32.dll

extern GetLastError
import GetLastError c:\Windows\system32\Kernel32.dll

extern SetConsoleWindowInfo
 ;SetConsoleWindowInfo PROTO,     ; set position of console window
   ; nStdHandle:dword,            ; screen buffer handle
   ; bAbsolute:dword,             ; coordinate type
   ; pConsoleRect:PTR SMALL_RECT  ; ptr to window rectangle
import SetConsoleWindowInfo c:\Windows\system32\Kernel32.dll

extern SetConsoleTitleA      ; set console window title
;    pString:PTR BYTE        ; points to string
;Returns: (BOOL) eax = TRUE (non-zero) if successful,
;                      FALSE (zero) if fails.
import SetConsoleTitleA c:\Windows\system32\Kernel32.dll

extern GetStdHandle
  ; :dword
  ;   STD_INPUT_HANDLE
  ;   STD_OUTPUT_HANDLE
  ;   STD_ERROR_HANDLE
import GetStdHandle c:\Windows\system32\Kernel32.dll

extern SetConsoleTextAttribute
;    nStdHandle:dword,   ; console output handle
;    nColor:dword        ; Specifies the foreground and background color attributes.
;Returns: (BOOL) eax = TRUE (non-zero) if successful,
;                      FALSE (zero) if fails.
;FOREGROUND_BLUE EQU 1h
; FOREGROUND_GREEN EQU 2h
; FOREGROUND_RED EQU 4h
; FOREGROUND_INTENSITY EQU 8h
; BACKGROUND_BLUE EQU 10h
; BACKGROUND_GREEN EQU 20h
; BACKGROUND_RED EQU 40h
; BACKGROUND_INTENSITY EQU 80h
import SetConsoleTextAttribute c:\Windows\system32\Kernel32.dll

extern WriteConsoleA
  ; hConsoleOutput : dword
  ;   handle of the console screen buffer to write to
  ; lpBuffer : dword
  ;   pointer to buffer to write from
  ; nNumberOfCharsToWrite : dword
  ; lpNumberOfCharsWritten : dword
  ;   pointer to 32-bit variable,
  ;   filled with number of characters written
  ; lpReserved : dword
  ;   Reserved. Must be NULL.
import WriteConsoleA c:\Windows\system32\Kernel32.dll

extern ExitProcess
  ; :dword
  ;   uExitCode
import ExitProcess c:\Windows\system32\kernel32.dll

; ************
; * INCLUDES *
; ************
;%include "F:\Assembleur\nasm\inc\win32n.inc"
;   This is Windows-related include file for NASM
;   by Tamas Kaproncai, available at
;   http://rs1.szif.hu/~tomcat/win32/

; ***********
; * EQUATES *
; ***********

;STD_OUTPUT_HANDLE EQU -11 ; This one is in Win32n.inc
                          ; but we avoid the inclusion of such
                          ; a large file, specially because of
                          ; the large listing it would generate
;STD_INPUT_HANDLE EQU -10
; ***************
; * DATAS       *
; ***************

SECTION DATA USE32 CLASS=data
Uni     							dd     1
Zero   								dq     0.0
Un     								dq     1.0
Deux   								dq     2.0
Trois   							dq     3.0
Cinq                  dq     5.0
Sept   								dq     7.0
Dix    								dq     10.0 
DixM    							dq     10000.0 
Q_div64  							dq     1000000000000000000.0
Lim_Nb								dq		 965211250482432409.0
     
Coordwin: 
 istruc  SMALL_RECT 
   at SMALL_RECT.left,   dw 0
   at SMALL_RECT.top,    dw 0
   at SMALL_RECT.right,  dw 79
   at SMALL_RECT.bottom, dw 40
 iend

FileOverlapped:
 istruc OVERLAPPED
   at OVERLAPPED.Internal,        dd 0 ; set by system after call
   at OVERLAPPED.InternalHigh,    dd 0 ; set by system after call
   at OVERLAPPED.loffset,         dd 0xFFFFFFFF  ;
   at OVERLAPPED.OffsetHigh,      dd -1
   at OVERLAPPED.hEvent,          dd 0
;Structure Overlapped for writing at the end of the file

pFilename       db  'Primes_File.csv',0  
 	
Msg_Console 					times 50 db ' '
Lg_Msg_Console 				EQU 50

Sep             db  ';',0
Lg_Sep    EQU $-Sep

Ret_File        db  0Dh,0Ah,0
Lg_Ret_File  EQU $-Ret_File    

Console_Title   db  'Decomposition of primes',0  

Ligne           db  0Ah,0Dh,0
Lg_Ligne EQU $-Ligne

Espace          db  ' ',0
Lg_Espace EQU $-Espace

Msg_Init1       db	' This program give you the decomposition of the prime number you enter : ',0
Lg_Msg_Init1 EQU $-Msg_Init1

Msg_Init2       db	' Prime = Weight * Level + Jump',0
Lg_Msg_Init2 EQU $-Msg_Init2

Msg_Init2a      db	' Only trivial division',0
Lg_Msg_Init2a EQU $-Msg_Init2a

Msg_Init3       db	' Tested up to 948895995818594437 in 664 sec. with',0
Lg_Msg_Init3 EQU $-Msg_Init3

Msg_Init31       db	' Intel Core i7 860 2,80Ghz - 3 Go RAM',0
Lg_Msg_Init31 EQU $-Msg_Init31

Msg_Init3a      db	' Fabien Sibenaler 2007 - 2012',0Dh,0Ah,0
Lg_Msg_Init3a EQU $-Msg_Init3a

Msg_Init3b      db  ' Classification of primes by Remi Eismann',0
Lg_Msg_Init3b EQU $-Msg_Init3b

Msg_Init3c      db  ' http://reismann.free.fr/',0
Lg_Msg_Init3c EQU $-Msg_Init3c

Msg_Init4       db	' ( CTRL-C to exit )',0
Lg_Msg_Init4 EQU $-Msg_Init4

Msg_Accueil     db	'Enter a prime number  p < 965,211,250,482,432,384  : ',0
Lg_Msg_Accueil EQU $-Msg_Accueil

Msg_Cramer      db  'Cramer limit :',0
Lg_Msg_Cramer EQU $-Msg_Cramer

Msg_Nombre      db  'Number     :',0
Lg_Msg_Nombre EQU $-Msg_Nombre

Msg_Nombre_pn1  db  'Number sup :',0
Lg_Msg_Nombre_pn1 EQU $-Msg_Nombre_pn1

Msg_Max         db  'Number too high !!!',0Dh,0Ah,0
Lg_Msg_Max EQU $-Msg_Max

Msg_Calcul      db  'Running ...',0
Lg_Msg_Calcul EQU $-Msg_Calcul

Msg_Temps       db  'Time in ms :',0
Lg_Msg_Temps EQU $-Msg_Temps

Msg_Saut        db  'Jump       :',0
Lg_Msg_Saut EQU $-Msg_Saut

Msg_L           db  'L          :',0
Lg_Msg_L EQU $-Msg_L

Msg_Poids       db  'Weight     :',0
Lg_Msg_Poids EQU $-Msg_Poids

Msg_Niveau      db  'Level      :',0
Lg_Msg_Niveau EQU $-Msg_Niveau

Msg_Nonnum      db  '!!! This is not a number !!!',0
Lg_Msg_Nonnum EQU $-Msg_Nonnum

Msg_Nonpre      db  '!!! This number is not prime !!!',0
Lg_Msg_Nonpre EQU $-Msg_Nonpre

Msg_N42         db  '!!! 2, 3, 7 cannot be decomposed !!!',0
Lg_Msg_N42 EQU $-Msg_N42

Msg_Pb_Create   db  '!!! Error CreateFile code : !!!',0
Lg_Msg_Pb_Create EQU $-Msg_Pb_Create

Msg_Pb_WriteF   db  '!!! Error WriteFile code : !!!',0
Lg_Msg_Pb_WriteF EQU $-Msg_Pb_WriteF

Msg_Header_File db  'Prime(n);Jump;Weight;Level;Time ms',0Dh,0Ah,0
Lg_Msg_Header_File EQU $-Msg_Header_File

Lg_Msg_Out EQU 20
Max EQU  2145390523

NON_PRIME       db '0' 
PRIME           db '1'


; ----------------------------------
; The 67,000,000th prime is 1,337,195,521.

SECTION DATA? USE32 CLASS=bss
test_file_hand        resd  1
top_prime             resd  1
tabpre 								resd 	50000000 
; limite p(50 000 000)^2 =  (982451653)^2 = 965 211 250 482 432 409
qdividende						resq  1
qpuiss    						resq  1
qword_alim            resq  1 
zeroq									resq  1
rec_poids							resq  1
rec_niveau  					resq  1
nb_a_teste						resq  1
qdiv									resq  1
nombre_cons  					resq  1
nb_Cram   						resq  1
accu_qw   						resq 	1
mult   								resq  1
reste  								resq  1
accu_dw   						resd 	1
multd  								resd  1
NumberOfCharsRead     resd 	1	
NumberOfCharsWritten 	resd 	1
NumberOfCharsWrittenF	resd 	1
V_Eax                	resd 	1
lim   								resd 	1
lim_pn1								resd 	1
lim_Poids 						resd 	1
lim_Niveau 						resd 	1
pos   								resd 	1
nbpre 								resq	1
tempsi 								resd  1
tempsf 								resd  1
saut    							resq  1
p_n   								resq  1 
p_n1   								resq  1 
l   									resq  1
dif_saut							resd  1
poids   							resq  1
niveau   							resq  1
nb_chiffres  					resd  1
nombre_lim  					resd  1
top_p  								resd  1
test_num   						resd  1
lg_tabpre							resd 	1
max 									resd 	1
Msg_Out  							resb 	10 
fcw16									resw  1
fcw16_save						resw  1
lim_Cram   						resd  1



; -----------------------------------
; ***************
; * 	 CODE     *
; ***************

SECTION CODE USE32 CLASS=CODE
..start:
;------------------------- Macros -------------------------------------
%macro Write_Msg 2
	pushad	
  push dword STD_OUTPUT_HANDLE
  call [GetStdHandle]    ; -> eax = handle of the
                         ; active screen buffer
  push dword 0
  push dword NumberOfCharsWritten
  push dword %1
  push dword %2
  push eax
  call [WriteConsoleA]
	popad
%endmacro

%macro Write_File 2
	pushad	
  push dword FileOverlapped
	push dword NumberOfCharsWrittenF
	push dword %1
	push dword %2
	push dword [test_file_hand]
	call [WriteFile]
  call Test_Write
	popad
%endmacro

;------------------------ OpenFile  -----------------------------------------------
	pushad
	push dword 0 ; no template for the file
	push dword FILE_ATTRIBUTE_NORMAL
	push dword 4 ; create only if not existexist
	push dword 0 ; can not inherit
	push dword 0 ;  can not share 
	push dword GENERIC_WRITE ; access mode
	push dword pFilename ; file name
	call [CreateFileA]
	mov [test_file_hand], eax
	call [GetLastError]
	cmp eax,183
	je dim_console
	popad
	Write_File Lg_Msg_Header_File, Msg_Header_File
	je dim_console
	
;------------------------ Dimensions console   -----------------------------------------
dim_console:
  pushad	
  push dword STD_OUTPUT_HANDLE
  call [GetStdHandle]    ; -> eax = handle                       
  push dword Coordwin
  push dword Uni
  push eax
  call [SetConsoleWindowInfo]
	popad
	
;------------------------ Title console  -----------------------------------------------	
	pushad
	push dword Console_Title
	call [SetConsoleTitleA]
	popad
	
;------------------------ Welcome message  ---------------------------------------------	
	Write_Msg Lg_Ligne, Ligne
	Write_Msg Lg_Msg_Init1, Msg_Init1
	Write_Msg Lg_Ligne, Ligne
	call Text_Bleu 
	Write_Msg Lg_Msg_Init2, Msg_Init2
	Write_Msg Lg_Ligne, Ligne
	call Text_Blanc
	Write_Msg Lg_Msg_Init2a, Msg_Init2a
	Write_Msg Lg_Ligne, Ligne
	Write_Msg Lg_Msg_Init3, Msg_Init3
	Write_Msg Lg_Ligne, Ligne
	Write_Msg Lg_Msg_Init31, Msg_Init31
	Write_Msg Lg_Ligne, Ligne
	Write_Msg Lg_Msg_Init3a, Msg_Init3a
	Write_Msg Lg_Ligne, Ligne
	call Text_Jaune
	Write_Msg Lg_Msg_Init3b, Msg_Init3b
	Write_Msg Lg_Ligne, Ligne
	Write_Msg Lg_Msg_Init3c, Msg_Init3c
	Write_Msg Lg_Ligne, Ligne
	call Text_Blanc
	Write_Msg Lg_Msg_Init4, Msg_Init4
	Write_Msg Lg_Ligne, Ligne
	
;------------------------ Start of main routine----------- ------------------------------		
debut_pgm:
  mov dword [top_prime], 0	
	Write_Msg 	Lg_Ligne, Ligne
	Write_Msg 	Lg_Msg_Accueil, Msg_Accueil
	Write_Msg 	Lg_Ligne, Ligne
	
;------------------------Read the number p(n) from the console ------------------------------	
	pushad	
  push dword STD_INPUT_HANDLE
  call [GetStdHandle]    ; -> eax = handle of the
                         ; active screen buffer
  push dword 0
  push dword NumberOfCharsRead
  push dword Lg_Msg_Out
  push dword Msg_Out
  push eax
  call [ReadConsoleA]
	popad
	Write_Msg Lg_Ligne, Ligne
	
;-----------------------Get the number of millisecondes since start of PC ---------------	
	xor eax, eax
	call [GetTickCount]
	mov [tempsi], eax
	
;-----------------------Get the number of chars read ------------------------------------	
	xor eax, eax
	mov eax, [NumberOfCharsRead]
	dec eax
	dec eax
	mov [nb_chiffres], eax

;-----------------------Verify numericity of p(n) -----------------------------------------
	xor edi, edi
	mov ecx, [nb_chiffres]
boucle_num:
	cmp ecx, 0
	je conversion_entier
	dec ecx 
	xor eax, eax
	mov al, [Msg_Out+edi]
	inc edi
	mov bl, 47
	mov dword [test_num], 0
boucle_test:
	inc bl
	cmp bl, 58
	je non_numerique
	cmp al, bl
	jne boucle_test
	mov dword [test_num], 1
	jmp boucle_num
non_numerique:
	Write_Msg Lg_Msg_Nonnum, Msg_Nonnum 
	Write_Msg Lg_Ligne, Ligne
	jmp fin_pgm
	
;------------------------Convert p(n) in integer qword ------------------------------------
conversion_entier:
	mov dword [nombre_cons], 0
	mov dword [nombre_cons+4], 0
	mov dword [mult], 10	
	mov dword [nombre_cons], 0
	xor edi, edi
	mov ecx, [nb_chiffres]
boucle_dec:	
	dec ecx
	cmp ecx, 0
	jz fin_boucle_dec
	xor eax, eax
	mov al, [Msg_Out+edi]
	sub al, 48
	mov dword [accu_qw], 0
	mov dword [accu_qw+4], 0
	mov [accu_qw], al
	push ecx
boucle_puiss:
	finit
	fild qword [accu_qw]
	fild qword [mult]
	fmul st1
	fistp qword [accu_qw]
	dec ecx
	cmp ecx, 0
	jnz boucle_puiss
	pop ecx
	fild qword [accu_qw]
	fild qword [nombre_cons]
	fadd st0, st1
	fistp qword [nombre_cons]
	inc edi
	jmp boucle_dec
	
fin_boucle_dec:	
	xor eax, eax
	mov al, [Msg_Out+edi]
	sub al, 48
	add [nombre_cons], eax
	
;-------------------------The case of 1, 2, 3, 7          --------------------------------------------------	
;-------------------------            1 * 2 * 3 * 7  = 42 --------------------------------------------------
  finit
  fild qword [nombre_cons]
  fld  qword [Un]
  fcomi st0, st1
  jne case_of_237
  Write_Msg Lg_Msg_Nonpre, Msg_Nonpre
	Write_Msg Lg_Ligne, Ligne
	jmp fin_pgm		
	
case_of_237:
	fild qword [nombre_cons]
  fld  qword [Deux]
  fcomi st0, st1
  je nb_42	
  
  fild qword [nombre_cons]
  fld  qword [Trois]
  fcomi st0, st1
  je nb_42
  
  fild qword [nombre_cons]
  fld  qword [Sept]
  fcomi st0, st1
  je nb_42	
  			
	jmp case_of_5

nb_42:				
	Write_Msg Lg_Msg_N42, Msg_N42
	Write_Msg Lg_Ligne, Ligne
	jmp fin_pgm
	
;-------------------------The case of 5                 --------------------------------------------------
case_of_5:
  finit
  fild qword [nombre_cons]
  fld  qword [Cinq]
  fcomi st0, st1
  je nb_5	
  
  jmp max_value_pn
  
nb_5:	
  fld   qword [Cinq]
	fistp qword [p_n]
	fld   qword [Deux]
	fistp qword [saut]
	fld   qword [Trois]
	fistp qword [poids]
	fld qword [Un]
	fistp qword [niveau]			
	jmp affichage_fin	
		
;-------------------------Test the maximum value of p(n) --------------------------------------------------
max_value_pn:	
	finit
  fild qword [nombre_cons]
  fld  qword [Lim_Nb]
  fcomi st0, st1
  jbe nb_suplim			
	jmp calcul_borne
	
nb_suplim:
	Write_Msg Lg_Msg_Max, Msg_Max 
	Write_Msg Lg_Ligne, Ligne
	jmp fin_pgm
	

;------------------------ lim sup of p(n+1) Cramer's conjecture  ----------------------------------------
calcul_borne:	
	finit
	FLDLN2                     	; load of ln(2)-> y 
	fild  qword [nombre_cons]   	; x
	fyl2x												; ln(2) * log2(x) -> ln(x)
	fmul st0										; ln(x)^2									
	fld   qword [Un]
	fadd st0, st1
	fild  qword [nombre_cons]
	fadd st0, st1
	fistp qword [nb_Cram]
		
;-----------------------limit of primality test  : ent(racine(sup(p(n+1))) + 1 = limit of Cramer
;-----------------------where sup(p(n+1)) calculated with Cramer's conjecture ------ 
	finit
	fild  qword [nb_Cram] 
	fsqrt
	fld   qword [Un]
	fadd st0, st1 
	fistp qword [lim_Cram]
	  
;--------------------- compute prime numbers up to limit of  Cramer and primality test on pn --------	
	Write_Msg Lg_Msg_Calcul, Msg_Calcul 
	Write_Msg Lg_Ligne, Ligne
	Write_Msg Lg_Ligne, Ligne
	mov dword [zeroq], 0
	mov dword [zeroq], 0
	mov dword [qdiv], 2
	mov dword [qdiv+4], 0
	
	finit
	fild  qword [qdiv]
	fild  qword [nombre_cons] 
	fprem1 
	fistp qword [reste]
	mov eax, [reste]
	cmp dword eax, 0
	je nombre_cons_compose

	mov dword [zeroq], 0
	mov dword [zeroq], 0
	mov dword [qdiv], 3
	mov dword [qdiv+4], 0
	finit	
	fild qword [qdiv]
	fild qword [nombre_cons] 
	fprem1 
	fistp qword [reste]
	mov eax, [reste]
	cmp dword eax, 0
	je nombre_cons_compose
	
	pushad
	mov dword [top_p], 0
	mov dword [tabpre],2
	mov dword [tabpre+4],3
	mov dword eax, 4 
	mov edi, 1
boucle_princ_nbpre:
	inc eax
	cmp eax , [lim_Cram]
	je fin_calcul_p
	call CALCUL_LIM	
	mov ebx, 2	
	mov esi, 0
;------------------------divisions of eax by successive prime numbres in dword --------	
boucle_nbpre:
	push eax
	xor edx, edx
	div ebx
	pop eax
	sub edx, 0
	jz non_premier
	cmp ebx, [lim]
	jge  nb_premier
	inc esi
	mov ebx, [tabpre+4*esi]
	jmp boucle_nbpre
;------------------------ eax is prime, we load it in the table and in qdiv ----------
nb_premier:
	inc edi
	mov [tabpre+4*edi],eax
	mov dword [qdiv], 0
	mov dword [qdiv+4], 0
	mov dword [qdiv], eax
	;--------------------- test of primality of pn -------------------------------------
	finit	
	fild qword [qdiv]
	fild qword [nombre_cons] 
	fprem
	fistp qword [reste]
	push eax
	mov eax, [reste]
	cmp dword eax, 0
	je nombre_cons_compose
	pop eax
	jmp boucle_princ_nbpre
non_premier:
	jmp boucle_princ_nbpre
	
nombre_cons_compose:
	Write_Msg Lg_Msg_Nonpre, Msg_Nonpre 
	Write_Msg Lg_Ligne, Ligne
	jmp fin_pgm
	
fin_calcul_p:	
	mov eax, [tabpre+4*edi]
		
;----------------------p(n) is prime, we compute p(n+1) ------------------------------	
	finit
	fild  qword [nombre_cons]   	; x	
	fistp qword [nb_a_teste]
boucle_princ_pn1:
	finit
	fild  qword [nb_a_teste]   	; x	
	fld   qword [Deux]
	fadd st0, st1
	fistp qword [nb_a_teste]
	call CALCUL_LIM_Q
						
	mov ebx, 2	
	mov esi, 0
	
;--------------------- divisions of number to test by successive prime numbers in dword--------	
boucle_pn1:
	mov dword [qdiv], 0
	mov dword [qdiv+4], 0
	mov dword [qdiv], ebx
	finit	
	fild qword [qdiv]
	fild qword [nb_a_teste] 
	fprem
	fistp qword [reste]
	mov eax, [reste]
	cmp dword eax, 0
	je pn1_non_premier
		
	cmp ebx, [lim_pn1]
	jge  pn1_premier
	inc esi
	cmp esi, edi
	jg pn1_premier
	mov ebx, [tabpre+4*esi]
	jmp boucle_pn1
pn1_premier:
	finit
	fild  qword [nb_a_teste]   	; x	
	fistp qword [p_n1]
	mov eax, [p_n1]
	jmp classification
pn1_non_premier:
	jmp boucle_princ_pn1
	
;-----------------------we found p(n+1) : classification of p(n) ------------------------------		
classification:	
	
	finit
	fild  qword [nombre_cons]   	
	fistp qword [p_n]
	finit
	fild  qword [nb_a_teste]   		
	fistp qword [p_n1]
				
;--------------------- the jump ------------------------------			
	
	mov eax, [p_n1]
	finit
	fild  qword [p_n] 
	fild  qword [p_n1]
	fsub st0, st1  	
	fistp qword [saut]
	
;---------------------compute L = 2*p(n) - p(n+1) ------------------------	
	finit
	fild  qword [p_n] 
	fld   qword [Deux]
	fmul st0, st1  
	fistp qword [l]
	
	finit	
	fild  qword [p_n1]
	fild  qword [l]
	fsub st0, st1
	fistp qword [l]
	
	finit
	fild qword [saut]
	fild qword [l]
	fcomi st0, st1	
	jg l_not0                     ; l not = 0
	mov dword [l], 0              ; l = 0
	mov dword [l+4], 0
	jmp calcul_limite_poids
	
l_not0:
;---------------------------------------------------------------------
;--------------- SIEVE BY WEIGHT -------------------------------------
;---------------------------------------------------------------------	
;------- Limit to find the weight ---
; int(sqrt(l))
calcul_limite_poids:
	finit
	fild  qword [l] 
	fsqrt
	fistp qword [lim_Poids]
	
;------- initialization of the weight = 1
	finit
	fld   qword [Un]
	fistp qword [rec_poids]

;-------- loop to find the weight
boucle_rec_poids:
;-------- weight +2
	finit
	fild  qword [rec_poids] 
	fld   qword [Deux]
	fadd st0, st1 
	fistp qword [rec_poids]

;-------- compare with the weight limit 
	finit
  fild qword [rec_poids]
  fild qword [lim_Poids]
  fcomi st0, st1
  jb fin_rec_poids        ; correction <
  
;-------- compare with the jump 
  finit
  fild qword [saut]
  fild qword [rec_poids]
  fcomi st0, st1
  jbe boucle_rec_poids
  
 ;------- is the right weight ?
 	finit	
	fild  qword [rec_poids]
	fild  qword [p_n] 
	fprem
	fistp qword [reste]
	
	finit
  fild qword [reste]
  fild qword [saut]
  fcomi st0, st1
;--- yes
  je poids_trouve
;--- no
	jmp boucle_rec_poids

;----- we find the weight !
poids_trouve:
	fild  qword [rec_poids]
	fistp qword [poids]
;-----we compute the level and that's all !
	finit
	fild  qword [poids]
	fild  qword [l]
	fdiv st0, st1
	fistp qword [niveau]
	jmp affichage_fin

;---- we don't find the find, let's sieve by level 
fin_rec_poids:	 		
	jmp calcul_limite_niveau
	
;-------------------------------------------------------------------------
;-------------------- SIEVE by LEVEL   -----------------------------------
;--------------------------------------------------------------------------
;----- limit to find the level
;----- = jump - 1	
calcul_limite_niveau:
	finit
	fld  qword [Un]
	fild qword [saut] 
	fsub st0, st1 
	fistp qword [lim_Niveau]
	
;----- test of parity of the level limite
	finit	
	fld   qword [Deux]
	fild  qword [lim_Niveau]
	fprem
	fistp qword [reste]
	
	finit
  fild qword [reste]
  fild qword [Zero]
  fcomi st0, st1
;--- limit is even, we go to add 1
  je increment_lim_niveau
;--- limit is odd, it's good
	jmp init_rec_niveau
	
;----- we add 1 so that the level limit is odd
increment_lim_niveau:
	finit
	fild  qword [lim_Niveau]
	fld   qword [Un]
	fadd st0, st1 
	fistp qword [lim_Niveau]
	
;------- initialization of the level :  limit of level + 2
init_rec_niveau:
	finit
	fild  qword [lim_Niveau]
	fld   qword [Deux]
	fadd st0, st1 
	fistp qword [rec_niveau]
	
;-------- loop to find the level
boucle_rec_niveau:
;-------- subtract level by 2
	finit
	fld   qword [Deux]
	fild  qword [rec_niveau] 
	fsub st0, st1 
	fistp qword [rec_niveau]
	
;-------- compare level with 1 
	finit
  fld  qword [Un]
  fild qword [rec_niveau]
  fcomi st0, st1
  jbe  fin_rec_niveau
  
;-------- if the level  divide L
  finit
  fild  qword [rec_niveau] 
	fild  qword [l]
	fprem 
	fistp qword [reste]
		
	finit
  fild qword [Zero]
  fild qword [reste]
  fcomi st0, st1
  jne boucle_rec_niveau
 ;------- is the right level ?
 	finit
	fild qword [rec_niveau]
	fild qword [l]
	fdiv st0, st1
	fistp qword [poids]
	
 	finit	
	fild qword [poids]
	fild qword [p_n] 
	fprem
	fistp qword [reste]
	
	finit
  fild qword [reste]
  fild qword [saut]
  fcomi st0, st1
;--- yes
  je niveau_trouve
;--- no
	jmp boucle_rec_niveau

;----- that's all 
niveau_trouve:
	finit
	fild qword [rec_niveau]
	fistp qword [niveau]
	jmp affichage_fin 

;---- we don't find the level so weight = l, level = 1	
fin_rec_niveau:	
	
	finit
	fild qword [l]
	fistp qword [poids]	 
	finit
	fld qword [Un]
	fistp qword [niveau]  		
	jmp affichage_fin

;--------------------------------End of classification of  p(n) --------------------------------------
;--------- Display results on console and write then to file -----------------------------------------	

affichage_fin:
	mov dword [top_prime], 1
;----------------- display p(n)
	Write_Msg 	Lg_Msg_Nombre, Msg_Nombre
	finit
	fild  qword [p_n]
	fistp qword [qword_alim]
	call Alim_Msg_Console_Qword
	Write_Msg Lg_Msg_Console, Msg_Console
	Write_File 18, Msg_Console
	Write_File Lg_Sep, Sep
	Write_Msg Lg_Ligne, Ligne
	Write_Msg Lg_Ligne, Ligne
;----------------- display jump
	Write_Msg Lg_Msg_Saut, Msg_Saut 
	finit
	fild  qword [saut]
	fistp qword [qword_alim]
	call Alim_Msg_Console_Qword
	Write_Msg Lg_Msg_Console, Msg_Console
	Write_File 18, Msg_Console
	Write_File Lg_Sep, Sep
	Write_Msg Lg_Ligne, Ligne
	Write_Msg Lg_Ligne, Ligne
;----------------- display weight
	Write_Msg Lg_Msg_Poids, Msg_Poids 
	finit
	fild  qword [poids]
	fistp qword [qword_alim]
	call Alim_Msg_Console_Qword
	Write_Msg Lg_Msg_Console, Msg_Console
	Write_File 18, Msg_Console
	Write_File Lg_Sep, Sep
	Write_Msg Lg_Ligne, Ligne
	Write_Msg Lg_Ligne, Ligne 
;----------------- display level
	Write_Msg Lg_Msg_Niveau, Msg_Niveau 
	finit
	fild  qword [niveau]
	fistp qword [qword_alim]
	call Alim_Msg_Console_Qword
	Write_Msg Lg_Msg_Console, Msg_Console
	Write_File 18, Msg_Console
	Write_File Lg_Sep, Sep
	Write_Msg Lg_Ligne, Ligne
	Write_Msg Lg_Ligne, Ligne 
fin_pgm:
;----------------- time of running
	xor eax, eax
	call [GetTickCount]
	mov [tempsf], eax
	pushad 
	Write_Msg Lg_Ligne, Ligne
	Write_Msg Lg_Msg_Temps, Msg_Temps
	xor eax, eax
	xor ebx, ebx
	mov eax, [tempsf]
	mov ebx, [tempsi]
	sub eax, ebx
	call Alim_Msg_Console_Eax
	Write_Msg Lg_Msg_Console, Msg_Console
	cmp dword [top_prime], 0
	je last_proc
	Write_File 18, Msg_Console
	Write_File Lg_Ret_File, Ret_File
last_proc:
	Write_Msg Lg_Ligne, Ligne
	popad
	jmp debut_pgm
	
	push dword 0           ; exit code
  call [ExitProcess]
	
;********************************************************************
;	                       PROCEDURES
;********************************************************************
Test_Write
;------------------------ Attributes Text   -----------------------------------------
	cmp eax, 0
	jne writeok
	Write_Msg Lg_Msg_Pb_WriteF, Msg_Pb_WriteF
	call [GetLastError]
	call Alim_Msg_Console_Eax
	Write_Msg Lg_Msg_Console, Msg_Console
  writeok:
  ret
  
Text_Jaune
;------------------------ Attributes Text   -----------------------------------------
  pushad	
  push dword STD_OUTPUT_HANDLE
  call [GetStdHandle]    ; -> EAX = handle of the                      
  push dword FOREGROUND_GREEN | FOREGROUND_RED ; jaune
  push eax
  call [SetConsoleTextAttribute]
	popad
	ret 
	
Text_Blanc
;------------------------ Attributes Text   -----------------------------------------
  pushad	
  push dword STD_OUTPUT_HANDLE
  call [GetStdHandle]    ; -> eax = handle                       
  push dword FOREGROUND_BLUE | FOREGROUND_GREEN | FOREGROUND_RED ; blanc
  push eax
  call [SetConsoleTextAttribute]
	popad
	ret 
	
Text_Bleu
	;------------------------ Attributes Text   -----------------------------------------
  pushad	
  push dword STD_OUTPUT_HANDLE
  call [GetStdHandle]    ; -> eax = handle of the                      
  push dword FOREGROUND_BLUE | FOREGROUND_GREEN ; bleu
  push eax
  call [SetConsoleTextAttribute]
	popad
	ret
	
;------------------------ Compute sqrt(nb) + 1 in qword   -----------------------------		
CALCUL_LIM_Q
	pushad
	finit
	fild qword [nb_a_teste] 
	fsqrt 
	fistp dword [lim_pn1]
	inc dword [lim_pn1]
	popad
	ret
	
;------------------------ Compute sqrt(nb) + 1 in dword   -----------------------------		
CALCUL_LIM
	pushad
	mov [lim],eax 
	finit
	fild dword [lim] 
	fsqrt 
	fist dword [lim]
	inc dword [lim]
	popad
	ret

;----------------------------------------------------------------------------------
; display of eax on console
;----------------------------------------------------------------------------------
Alim_Msg_Console_Eax:
		pushad	
		mov ecx, 49
		mov edi, 0
		
	init:	
		mov word [Msg_Console + edi], ' '
		inc edi
		loop init
		
		mov edi, 0
		mov ebx, 3B9ACA00h ;1000000000 en dec
		xor cl, cl
		
	boucle3:
		xor edx, edx
		div ebx
		;test to not display the begging zeros
		or cl, al
		cmp cl, 0
		je nepasafficherzero3
		push eax
		add al, 30h ;48 en dec
		mov ah, 0Eh
		mov [Msg_Console + edi], ax
		inc edi
		;int 10h
		pop eax
		
	nepasafficherzero3:
		cmp bl, 1
		je finboucle3
		push eax
		push edx
		xor edx, edx
		mov eax, ebx
		mov ebx, 0000000Ah ;divide by 10 soit 000A
		div ebx
		mov ebx, eax
		pop eax ;load edx in eax and eax in edx
		pop edx
		jmp boucle3
		
	finboucle3:
		cmp cl, 0
		jne fin3
		mov al, 30h
		mov [Msg_Console + edi], al
		jmp fin_alim
		
	fin3:
		mov word [Msg_Console + edi], ' '
		
	fin_alim:
		popad
		ret
		
;----------------------------------------------------------------------------------
; display of a qword on console
;----------------------------------------------------------------------------------
Alim_Msg_Console_Qword:
		pushad
		mov ecx, 49
		mov edi, 0
		
	init_q:	
		mov word [Msg_Console + edi], ' '
		inc edi
		loop init_q
		
		mov edi, 0
		finit
		fld qword [Q_div64]
		fistp qword [qpuiss]
		xor cl, cl
		
	boucle3_q:
		finit
		fstcw [fcw16]
		mov   ax, [fcw16]
		mov   [fcw16_save], ax
		and   ax, 0f0ffh
		or    ax, 00c00h
		mov   [fcw16], ax
		 
		fldcw  [fcw16]
		fild qword [qpuiss]
		fild qword [qword_alim]
		fdiv st0, st1
		FRNDINT
		fistp qword [qdividende]
		fldcw  [fcw16_save]
		xor eax, eax
		mov word ax, [qdividende]		
		or cl, al
		cmp cl, 0
		je nepasafficherzero3_q
		push eax
		mov word ax, [qdividende]
		add al, 30h ;48 in dec
		mov ah, 0Eh
		mov [Msg_Console + edi], ax	
		inc edi
		pop eax
				
	nepasafficherzero3_q:
		cmp dword [qpuiss], 1
		je finboucle3_q
		finit
		fild qword [qpuiss]
		fild qword [qword_alim]
		fprem
		fistp qword [qword_alim]
				
		finit
		fld qword [Dix]
		fild qword [qpuiss]
		fdiv st0, st1
		fistp qword [qpuiss]
		jmp boucle3_q
		
	finboucle3_q:
		cmp cl, 0
		jne fin3_q
		mov al, 30h
		mov [Msg_Console + edi], al
		jmp fin_alim_q
		
	fin3_q:
		mov word [Msg_Console + edi], ' '
		
	fin_alim_q:
		popad
		ret
